----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    21:48:20 06/08/2009 
-- Design Name: 
-- Module Name:    grafika - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity grafika is
    Port ( ALE : in  STD_LOGIC;
			  ALEn : out STD_LOGIC;
           RD : in  STD_LOGIC;
			  A16 : in STD_LOGIC;
           RAM_A : out  STD_LOGIC_VECTOR (16 downto 0);
           RAM_D : in  STD_LOGIC_VECTOR (7 downto 0);
           RAM_RD : out  STD_LOGIC;
           LCD_D : out STD_LOGIC_VECTOR(7 downto 0);
			  VDCLK : out STD_LOGIC;
			  LLCLK : out STD_LOGIC;
			  LFS : out STD_LOGIC);
end grafika;

architecture Behavioral of grafika is
	-- konstante
	constant SIRINA : integer := 160; -- = 640 / 4;
	constant VISINA : integer := 240; -- = 480 / 2;
	constant SIRINA_BITS : integer := 8;
	constant VISINA_BITS : integer := 8;
	constant RAM_BITS : integer := 17;
	
	-- stanje iscrtavanja
	type stanje_type is (prvibajt, drugibajt, novired);
	signal stanje : stanje_type := prvibajt;
	
	-- signali
	signal outlcda, outlcdb : STD_LOGIC_VECTOR((RAM_BITS - 2) downto 0); -- interna adresa za LCD RAM
	signal iLFS : STD_LOGIC := '0';
	signal iLLCLK : STD_LOGIC;
	signal w : STD_LOGIC_VECTOR(SIRINA_BITS - 1 downto 0);
	signal h : STD_LOGIC_VECTOR(VISINA_BITS - 1 downto 0);
	signal aclk, hclk, wclk, hrst, wrst : STD_LOGIC;
	
	-- registri
	signal framemod : STD_LOGIC_VECTOR(1 downto 0);	-- za shade-anje
	
	-- konverzije iz 2 bita boje u LCD pwm
	impure function color_to_bin(c : STD_LOGIC_VECTOR(1 downto 0)) return STD_LOGIC is
	begin
		case c is
			when "00" =>
				return '0';
			when "01" =>
				return framemod(0);
			when "10" =>
				return framemod(0) or framemod(1);
			when others =>
				return '1';
		end case;
	end function;
	
	-- komponente
	component acounter is generic (N : integer := 8; init : STD_LOGIC := '1');
    Port ( clk : in  STD_LOGIC;
           rst : in  STD_LOGIC;
           d : out  STD_LOGIC_VECTOR ((N-1) downto 0));
	end component;
begin

	LFS <= iLFS;
	LLCLK <= iLLCLK;
	ALEn <= not ALE;

	-- brojač za adresu u RAM-u
	aclk <= ALE when (stanje = prvibajt or stanje = drugibajt) else '0';
	addrcounter : acounter generic map (N => RAM_BITS - 1, init => '0')
		port map (clk => aclk, rst => hrst, d => outlcda);
	addrcounter2 : acounter generic map (N => RAM_BITS - 1, init => '0')
		port map (clk => aclk, rst => iLFS, d => outlcdb);
	
	-- brojač za širinu i visinu
	wclk <= (not ALE) when stanje = prvibajt else '1';
	wrst <= ALE when stanje = novired else '0';
	wcounter : acounter generic map (N => SIRINA_BITS, init => '0')
		port map (clk => wclk, rst => wrst, d => w);
	hclk <= (not ALE) when stanje = novired else '1';
	hcounter : acounter generic map (N => VISINA_BITS, init => '0')
		port map (clk => hclk, rst => hrst, d => h);

	-- display controller
	dispctrl : process (ALE, RAM_D)
	begin
		if (falling_edge(ALE)) then
			iLLCLK <= '0';
			hrst <= '0';
			
			case stanje is
				when prvibajt =>
					LCD_D(0) <= color_to_bin(RAM_D(1 downto 0));
					LCD_D(1) <= color_to_bin(RAM_D(3 downto 2));
					LCD_D(2) <= color_to_bin(RAM_D(5 downto 4));
					LCD_D(3) <= color_to_bin(RAM_D(7 downto 6));
					stanje <= drugibajt;
					VDCLK <= '1';
					iLFS <= '0';
					
				when drugibajt =>
--					LCD_D(4) <= color_to_bin(RAM_D(1 downto 0));
--					LCD_D(5) <= color_to_bin(RAM_D(3 downto 2));
--					LCD_D(6) <= color_to_bin(RAM_D(5 downto 4));
--					LCD_D(7) <= color_to_bin(RAM_D(7 downto 6));
					stanje <= prvibajt;
					VDCLK <= '0';
					
					if (w = SIRINA) then
						stanje <= novired;
						iLLCLK <= '1';
						
						if (h = 0) then
							framemod <= framemod + 1;
							iLFS <= '1';
						elsif (h = VISINA - 1) then
							hrst <= '1';
						end if;
					end if;
					
				when novired =>
					stanje <= prvibajt;
					
				when others =>
					stanje <= prvibajt;
			end case;
		end if;
	end process;
	
	
	-- pristup RAM-u
	ramacc : process (ALE, RD, A16, outlcda, RAM_D, 	stanje, outlcdb)
	begin
		if (ALE = '0') then
			RAM_A(15 downto 0) <= (others => 'Z');
			RAM_A(16) <= A16;
			RAM_RD <= RD;
		else
			RAM_RD <= '0';
			if (stanje = prvibajt) then
				RAM_A <= '0' & outlcda;
			else
--				if (h = VISINA - 1) then
					--RAM_A <= '1' & (outlcda + ((VISINA - 1) * SIRINA));
--				else
					RAM_A <= '1' & outlcdb;
--				end if;
			end if;
		
			LCD_D(4) <= color_to_bin(RAM_D(1 downto 0));
			LCD_D(5) <= color_to_bin(RAM_D(3 downto 2));
			LCD_D(6) <= color_to_bin(RAM_D(5 downto 4));
			LCD_D(7) <= color_to_bin(RAM_D(7 downto 6));
		end if;
	end process;

end Behavioral;
